'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createReduce = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _rx = require('rx');

var _rx2 = _interopRequireDefault(_rx);

var _utils = require('../utils');

var _errors = require('../errors');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var createWrapHookObservable = function createWrapHookObservable(map, pauser) {
  var wrapObservable = function wrapObservable(observable) {
    var memo = map.get(observable);
    if (memo) return memo;
    var wrapped = observable.doOnNext(function () {
      return pauser.onNext(false);
    }).doOnError(function () {
      return pauser.onNext(false);
    }).flatMapObserver(function (val) {
      return _rx2.default.Observable.just(val).doOnCompleted(function () {
        return pauser.onNext(true);
      });
    }, function (err) {
      return _rx2.default.Observable.create(function (o) {
        o.onError(err);
        pauser.onNext(true);
      });
    }).startWith(null).share();
    map.set(observable, wrapped);
    return wrapped;
  };
  return function (actions) {
    if (Array.isArray(actions)) {
      var _Rx$Observable;

      var observables = actions.map(_utils.getObservable).map(wrapObservable);
      return (_Rx$Observable = _rx2.default.Observable).combineLatest.apply(_Rx$Observable, _toConsumableArray(observables));
    } else {
      return wrapObservable((0, _utils.getObservable)(actions));
    }
  };
};

var createReduce = exports.createReduce = function createReduce(observable, nodeObservable, hookMap, pauser) {
  var hookSubject = new _rx2.default.ReplaySubject();
  var createHookObservable = createWrapHookObservable(hookMap, pauser);
  var connectReducer = function connectReducer(obs, setNextState, reducer) {
    obs.subscribeOnNext(function (_ref) {
      var _ref2 = _slicedToArray(_ref, 2);

      var vals = _ref2[0];
      var state = _ref2[1];

      var nextState = reducer(state, vals);
      if (typeof nextState === 'undefined') {
        (0, _errors.throwUndefinedNextStateError)();
      } else {
        setNextState(nextState);
      }
    });
  };

  var apiRealizationObservable = _rx2.default.Observable.combineLatest(nodeObservable.filter(function (node) {
    return node && !node.provisional;
  }), hookSubject, function (_ref3, _ref4) {
    var setNextState = _ref3.setNextState;
    var hookObservable = _ref4.hookObservable;
    var reducer = _ref4.reducer;

    var nextSubject = new _rx2.default.Subject();
    var nextObservable = nextSubject.withLatestFrom(observable);

    hookObservable.subscribe(nextSubject);
    connectReducer(nextObservable, setNextState, reducer);

    return true;
  });

  apiRealizationObservable.subscribeOnError(function (error) {
    throw error;
  });

  var reduce = function reduce(actions, reducer) {
    var hookObservable = createHookObservable(actions);

    hookSubject.onNext({ hookObservable: hookObservable, reducer: reducer });
    return makeApi();
  };

  var makeApi = function makeApi() {
    return {
      reduce: createReduce(observable, nodeObservable, hookMap, pauser)
    };
  };

  return reduce;
};